/****************** <lalVerbatim file="BlockHV">
Author: Tibbits, M. M.
$Id: Block.h,v 1.19 2003/07/06 20:06:08 jolien Exp $
********************************* </lalVerbatim> */

/******************************* <lalLaTeX file="BlockH">
\section{Header \texttt{Block.h}}
\label{s:Block.h}

Provides structures and definitions global to the block search,
notably \texttt{BlockSearchParams}.

\subsection*{Error conditions}

\input{BlockHErrTab}

\subsection*{Structures and Unions}

*************************************************** </lalLaTeX> */

#ifndef _BLOCK_H_
#define _BLOCK_H_

#include <math.h>
#include <mpi.h>
#include <lal/LALRCSID.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/SeqFactories.h>
#include <lal/LALConstants.h>
#include <lal/LALMoment.h>
#include <lal/Comm.h>
#include <lal/BlockRho.h>
#include <LALWrapperInterface.h>

/* C++ protection */
#ifdef  __cplusplus
extern "C" {
#endif

NRCSID( BLOCKH, "$Id: Block.h,v 1.19 2003/07/06 20:06:08 jolien Exp $");


/******** <lalErrTable file="BlockHErrTab"> ********/
#define BLOCKH_ENULL 1
#define BLOCKH_ENNUL 2
#define BLOCKH_EALOC 3
#define BLOCKH_ENUMZ 4
#define	BLOCKH_EARG  5
#define BLOCKH_EDATA 6

#define BLOCKH_MSGENULL "Null pointer"
#define BLOCKH_MSGENNUL "Non-null pointer"
#define BLOCKH_MSGEALOC "Memory allocation error"
#define BLOCKH_MSGENUMZ "Data segment length is zero"
#define	BLOCKH_MSGEARG  "Error parsing command-line arguments"
#define BLOCKH_MSGEDATA "Too few input data points to create useable search data"

/******** </lalErrTable> ********/

/* #define BLOCKSEARCH__DEBUG*/

/******************************* <lalLaTeX file="BlockH">

\subsubsection*{Union \texttt{ExchType}}
\index{\texttt{ExchType}}

\noindent
The enumerations are:\\
\\
PassData\\
GetData\\
PassResult\\
RePass\\
ExchFinished\\

*************************************************** </lalLaTeX> */

/******************************* <lalLaTeX file="BlockH">

\subsubsection*{Structure \texttt{BlockSearchParams}}
\index{\texttt{BlockSearchParams}}

\noindent
The fields are:\\
\\

*************************************************** </lalLaTeX> */

/*  Slice of one Level of wavelet decomposition  */
typedef struct
tagbandSeg
{
  REAL8Vector		*data;

  INT4			startTime;
  INT4			duration;

  INT2			algorithm;
  INT2			level;

  REAL8			rhoN;
  REAL8			value;
  INT4			index;

  REAL8			variance;
  REAL8			mean;

  struct bandSeg	*right;
  struct bandSeg	*left;
  struct bandSeg	*next;
}
bandSegment;

/*  Main Data Storage  */
/*  Contains all band segments from one time slice  */
typedef struct
tagBandObject
{
  bandSegment		**decomposition;
  gpsTimeInterval	times;

  INT4			sampleRate;
  INT2			levels;
  INT4			segmentID;

  /* Phase? */
}
bandObject;

/*  Result Object  */
typedef struct
tagBlockRes
{
  INT4			segmentID;
  INT2			bandLevel;

  INT4			startTime;
  INT4			duration;

  REAL8			rhoNStart;
  REAL8			rhoNEnd;
  REAL8			value;
  INT4			index;

  REAL8			variance;
  REAL8			mean;

  struct BlockRes	*next;
}
BlockResult;

/*  Main Parameter Structure  */
typedef struct
tagBlockSearchParams
{
  gpsTimeInterval	times;			/*  The GPS start and end time of the data   */

  bandSegment		*head;			/*  Head of Queue for Breadth First Search  */
  bandSegment		*tail;			/*  Tail of Queue for Breadth First Search  */
  bandObject		*data;			/*  Pointer to head of linked list  */

  BlockResult		*results;		/*  Result Objects Received from slave nodes  */
  BlockResult		*prevResults;		/*  Point to end of linked list  */

  dataBase		*resultsToStore;	/*  database results linked list (attached to output obj.)  */
  dataBase		*lastResultStored;	/*  database results linked list ( current->previous )  */

  REAL8			scaleExponent;		/*  Exponent for RhoN scaling  */
  REAL8			scaleFactor;		/*  Amplitude for RhoN scaling  */

  REAL8			*varianceThreshold;	/*  Scale factor on (SigmaZero)^2 to threshold event reporting  */
  REAL8			*varianceZero;		/*  The variance for the entire data segment passed in  */

  REAL8			*meanThreshold;		/*  Scale factor on (SigmaZero)^2 to threshold event reporting  */
  REAL8			*meanOffset;		/*  Scale factor on (MeanZero)^2 to threshold event reporting  */
  REAL8			*meanZero;		/*  The average background  */

  REAL8			*threshold;		/*  Threshold value for recording an event  */

  UINT4			marginOfExclusion;	/*  Seconds at beginning and end of data stream to ignore results from  */
  UINT4			numberOfSequences;	/*  Number of Sequences in data  */
  UINT4			segmentsRemaining;	/*  Number of Segments left to receive  */
  UINT4			resultsObtained;	/*  Number of Sequences that the Master has received results for  */
  UINT4			eventCounter;		/*  Used to store eventIDs in the database  */
  UINT4			numSlaves;		/*  Number of friends in Comm World  */
  UINT4			numOutput;		/*  Stores number of points in params->resultsToStore  */
  UINT4			overlap;		/*  Seconds at end of previous sample to include at beginning of next sample  */

  INT4			*sampleRate;		/*  Array to store the different sample rates of each input sequence  */

  INT4			minVectorLength;	/*  Boolean, To stop or not to stop?  */
  INT4			maxVectorLength;	/*  Parameter to tell the search to chop segments  */
  INT4			numberResults;		/*  Number of Result Objects in Linked List  */
  INT4			searchMaster;		/*  Boolean, am I in charge?  */
  INT4			rank;			/*  node number in Comm World  */

  INT2			recursiveSearch;	/*  Switch to change how recursion works  */
  INT2			overlapMethod;		/*  Switch to change overlap processing  */

  CHAR			channel[dbNameLimit];	/*  channel of data origin  */
  CHAR			ifo[dbNameLimit];	/*  detector of data origin  */
  CHAR			id[dbNameLimit];	/*  ID tag for easy Database Retrieval  */
}
BlockSearchParams;


/*  Identifiers for MPI messages types  */
typedef enum
{
  PassData,
  GetData,
  PassResult,
  RePass,
  ExchFinished
}
ExchType;

/*  Non-LAL Function Prototypes  */

void BlockDistributeDataMaster
(
 LALStatus             *status,
 LALSearchInput        *inout,
 void                  *searchParams,
 LALMPIParams          *mpiParams
);

void BlockDistributeDataSlave
(
 LALStatus             *status,
 LALSearchInput        *inout,
 void                  *searchParams,
 LALMPIParams          *mpiParams
);

void BlockParseData
(
 LALStatus	*status,
 LALSearchInput	*inout,
 void		*searchParams
);

void ApplySearchMaster
(
 LALStatus		*status,
 LALSearchOutput	*output,
 LALSearchInput		*input,
 LALApplySearchParams	*params
);

void ApplySearchSlave
(
 LALStatus		*status,
 LALSearchOutput	*output,
 LALSearchInput		*input,
 LALApplySearchParams	*params
);

void BuildOutput
(
 LALStatus		*status,
 LALSearchOutput	*output,
 BlockSearchParams	*params
);

void AddResults
(
 LALStatus		*status,
 LALSearchOutput	*output,
 BlockSearchParams	*params
);

/* Necessary Search Tree Functions  */
void BlockAddQ
(
 LALStatus	*status,
 bandSegment	*newSegment,
 void		*searchParams
);

bandSegment* BlockDelQ
(
 void		*searchParams
);

void BlockSplitLevel
(
 LALStatus	*status,
 bandSegment	*input,
 void		*searchParams
);

/* C++ protection */
#ifdef  __cplusplus
}
#endif

/******************************* <lalLaTeX file="BlockH">
  
\vfill{\footnotesize\input{BlockHV}}

\newpage\input{BlockInitSearchC}
\newpage\input{BlockConditionDataC}
\newpage\input{BlockApplySearchC}
\newpage\input{BlockFinalizeSearchC}

*************************************************** </lalLaTeX> */

#endif /* _BLOCK_H_ */
