/*----------------------------------------------------------------------- 
 * 
 * File Name: InitSearch.c
 *
 * Author: Brown, D. A. and Creighton, T. D.
 * 
 * Revision: $Id: InitSearch.c,v 1.48 2003/07/23 16:16:41 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#if 0
<lalVerbatim file="InitSearchCV">
Author: Brown, D. A. and Creighton, T. D.
$Id: InitSearch.c,v 1.48 2003/07/23 16:16:41 duncan Exp $
</lalVerbatim>

<lalLaTeX>
\subsection{Module \texttt{InitSearch.c}}
\label{inspiral:InitSearch.c}
Parses the \texttt{-filterparams} passed by the wrapperAPI and 
performs initialization and memory allocation of the \texttt{findchirp}
functions used in the \texttt{inspiral} shared object.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{InitSearchCP}

\subsubsection*{Description}
The \texttt{LALInitSearch()} function first checks that it has been 
called with reasonable argumnets and that the structure of the
\texttt{-filterparams} argument passes as \verb|initSearchParams->argv|
has the correct number of arguments and that each argument is within the range
of allowed values.

It then allocates memory for the global \texttt{InspiralSearchParams} 
structure which is passed between functions as the \verb|searchParams| 
argument.

Each member of the \texttt{InspiralSearchParams} structure is then populated.
\texttt{InspiralMasterParams} and \texttt{InspiralSlaveParams} are only
populated on the search master and search slaves respectively. The necessary 
initialisation functions from \texttt{findchirp} are called to populate the
\texttt{InspiralSlaveParams} structure.

If a template bank is requested to be generated using the functions in the
LAL \texttt{bank} package, the search master sets up the bank creation
parametsrs and calls \texttt{LALFindChirpCreateInspiralBank()} to generate a 
flat or heirarchical template bank, as approriate. If templates are specified
on the command line \texttt{InspiralTmpltsFromCmdLine()} is called to parse
these and create the bank. Only a flat bank can be specified by templates on
the command line.

The search slaves then allocate memory for the input data for use by 
\texttt{LALConditionData()}

\subsubsection*{Algorithm}

None.

\subsubsection*{Uses}
\begin{verbatim}
LALCalloc()
LALCreateFindChirpSegmentVector()
LALFindChirpSPDataInit()
LALFindChirpSPTemplateInit()
LALFindChirpFilterInit()
LALCreateFindChirpInput()
LALFindChirpChisqVetoInit()
LALCreateRandomParams()
LALFindChirpCreateInspiralBank()
LALCreateDataSegmentVector()
\end{verbatim}

\subsubsection*{Notes}

Memory for the input data is currently created using
\texttt{LALCreateDataSegmentVector()}. This is wasteful of memory and should
be improved.

\vfill{\footnotesize\input{InitSearchCV}}
</lalLaTeX>
#endif

#include "InitSearch.h"

#ifdef INSPIRAL_SO_ATTACH_DDD
#include <unistd.h>
#endif

NRCSID( INITSEARCHC, PKG "$Id: InitSearch.c,v 1.48 2003/07/23 16:16:41 duncan Exp $" );

/* <lalVerbatim file="InitSearchCP"> */
void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
/* </lalVerbatim> */
{
  InspiralSearchParams         *params          = NULL;
  InspiralMasterParams         *masterParamsPtr = NULL;
  InspiralSlaveParams          *slaveParamsPtr  = NULL;
  FindChirpInitParams          *initParamsPtr   = NULL;

  INITSTATUS( status, "LALInitSearch", INITSEARCHC );
  ATTATCHSTATUSPTR( status );

  
  /*
   *
   * sleep to give me time to attach ddd
   *
   */


#ifdef INSPIRAL_SO_ATTACH_DDD
  {
   unsigned int doze = 10;
   pid_t myPID;
   myPID = getpid();
   fprintf( stdout, "pid %d sleeping for %d seconds\n", myPID, doze );
   fflush( stdout );
   sleep( doze );
   fprintf( stdout, "pid %d awake\n", myPID );
   fflush( stdout );
  }
#endif


  /*
   *
   * check input, output and parameters
   *
   */


  /* check that the parameter structure exists */
  if ( !initSearchParams )
  { 
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL ); 
  }

  /* and that seachParams is null */
  if ( *searchParams )
  { 
    ABORT( status, INITSEARCHH_ENNUL, INITSEARCHH_MSGENNUL ); 
  }

  /* check that the arguments exist and there are an acceptable number */
  if ( ! initSearchParams->argv )
  { 
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL ); 
  }
  if ( !( initSearchParams->argc == INSPARG_NBNK 
        || initSearchParams->argc == INSPARG_NCMD 
        || initSearchParams->argc == INSPARG_NILD ) )
  { 
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }

  /* check that the first argument is -filterparams */
  if ( strcmp( initSearchParams->argv[0], "-filterparams" ) )
  { 
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }


  /* 
   *
   * sanity check basic arguments
   *
   */


  /* number of points in a data segment */
  if ( atoi( initSearchParams->argv[INSPARG_NPTS] ) <= 0 )
  { 
    ABORT( status, INITSEARCHH_ENUMZ, INITSEARCHH_MSGENUMZ ); 
  }
  
  /* number of data segments */
  if ( atoi( initSearchParams->argv[INSPARG_NSEG] ) <= 0 )
  { 
    ABORT( status, INITSEARCHH_ESEGZ, INITSEARCHH_MSGESEGZ ); 
  }
  
  /* data segment overlap */
  if ( atoi( initSearchParams->argv[INSPARG_OVRL] ) < 0 )
  {
    ABORT( status, INITSEARCHH_EOVLP, INITSEARCHH_MSGEOVLP ); 
  }


  /* 
   *
   * sanity check on filter parameters
   *
   */


  /* low frequency cutoff */
  if ( atof( initSearchParams->argv[INSPARG_FLOW] ) < 1.0 )
  {
    ABORT( status, INITSEARCHH_EFLOW, INITSEARCHH_MSGEFLOW ); 
  }
  /* inverse spectrum truncation in time domain */
  if ( atoi( initSearchParams->argv[INSPARG_TRNC] ) < 0 )
  {
    ABORT( status, INITSEARCHH_ETRNC, INITSEARCHH_MSGETRNC );
  }
  /* dynamic range */
  if ( atof( initSearchParams->argv[INSPARG_DYNR] ) < -126 ||
      atof( initSearchParams->argv[INSPARG_DYNR] ) > 127 )
  {
    ABORT( status, INITSEARCHH_EDYNR, INITSEARCHH_MSGEDYNR );
  }
  /* number of chisq bins */
  if ( atoi( initSearchParams->argv[INSPARG_NCHI] ) < 0 )
  { 
    ABORT( status, INITSEARCHH_EBINS, INITSEARCHH_MSGEBINS ); 
  }
  /* template bank match */
  if ( atof( initSearchParams->argv[INSPARG_MTCH] ) < 0 ||
      atof( initSearchParams->argv[INSPARG_MTCH] ) > 1 )
  { 
    ABORT( status, INITSEARCHH_EMTCH, INITSEARCHH_MSGEMTCH ); 
  }
  /* spec estimation */
  if ( atof( initSearchParams->argv[INSPARG_SPEC] ) > 4 )
  {
    ABORT( status, INITSEARCHH_ESPEC, INITSEARCHH_MSGESPEC ); 
  }


  /* 
   *
   * sanity check on engine parameters
   *
   */


  /* number of templates to exchange */
  if ( atoi( initSearchParams->argv[INSPARG_NTXC] ) <= 0 )
  { 
    ABORT( status, INITSEARCHH_ENTXC, INITSEARCHH_MSGENTXC ); 
  }

  /* rhosq thresholds: existance */
  if ( ! initSearchParams->argv[INSPARG_RHOT] )
  { 
    ABORT( status, INITSEARCHH_ERHOT, INITSEARCHH_MSGERHOT ); 
  }

  /* chisq threshold: existance */
  if ( ! initSearchParams->argv[INSPARG_CHIT] )
  {
    ABORT( status, INITSEARCHH_ECHIT, INITSEARCHH_MSGECHIT );
  }


  /* 
   *
   * sanity check on template input
   *
   */


  /* template source */
  if ( atoi( initSearchParams->argv[INSPARG_TSRC] ) < 0 ||
      atoi( initSearchParams->argv[INSPARG_TSRC] ) > 2  )
  { 
    ABORT( status, INITSEARCHH_ETSRC, INITSEARCHH_MSGETSRC ); 
  }

  /* template parameters */
  switch ( atoi( initSearchParams->argv[INSPARG_TSRC] ) )
  {
    case 0:
      /* templates from bank: check argc and existence of argument */
      if ( initSearchParams->argc != INSPARG_NBNK )
      {
        ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
      }
      if ( ! initSearchParams->argv[INSPARG_TPRM] )
      {
        ABORT( status, INITSEARCHH_ETPRM, INITSEARCHH_MSGETPRM ); 
      }
      break;

    case 1:
      /* templates from cmd: check argc and number of templates, templates */
      if ( initSearchParams->argc != INSPARG_NCMD )
      {
        ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
      }
      if ( atoi( initSearchParams->argv[INSPARG_TPRM] ) <= 0 )
      {
        ABORT( status, INITSEARCHH_ETMPZ, INITSEARCHH_MSGETMPZ );
      }
      if ( !initSearchParams->argv[INSPARG_TMPL] )
      {
        ABORT( status, INITSEARCHH_ETPRM, INITSEARCHH_MSGETPRM );
      }
      break;

    case 2:
      /* templates from ilwd: check argc */
      if ( initSearchParams->argc != INSPARG_NILD )
      {
        ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
      }
      break;

    default:
      /* should never get here, but just incase */
      ABORT( status, INITSEARCHH_ETPRM, INITSEARCHH_MSGETPRM );
      break;
  }


  /*
   *
   * allocate memory for the inspiral parameter structure
   *
   */


  *searchParams = params = 
    (InspiralSearchParams *) LALCalloc( 1, sizeof(InspiralSearchParams) ); 
  if ( !params )
  {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }
  
  initParamsPtr   = &(params->initParams);
  masterParamsPtr = &(params->masterParams);
  slaveParamsPtr  = &(params->slaveParams);

  /* record bank match for template generation */
  params->bankMatch = atof( initSearchParams->argv[INSPARG_MTCH] );


  /*
   *
   * populate InspiralSearchParams structure
   *
   */


  params->tmpltSource = 
    (UINT4) atoi( initSearchParams->argv[INSPARG_TSRC] );

  params->searchMaster = initSearchParams->nodeClass;

  /* if the debug flag is a single character parse it as an integer */
  if ( strlen( initSearchParams->argv[INSPARG_CRVC] ) == sizeof(CHAR) )
  {
    if ( ! strcmp( "0", initSearchParams->argv[INSPARG_CRVC] ) || 
        ! strcmp( "1", initSearchParams->argv[INSPARG_CRVC] ) ) 
    {
      params->inspiralDebugFlag = 
        (UINT4) atoi( initSearchParams->argv[INSPARG_CRVC] );
    }
    else
    {
      ABORT( status, INITSEARCHH_EDFLG, INITSEARCHH_MSGEDFLG );
    }
  }
  /* else parse the simulation type */
  else 
  {
    UINT4     crvcFlag;
    CHAR      junkBuffer[64];

    /* check that the argument string is not too long */
    if ( strlen( initSearchParams->argv[INSPARG_CRVC] ) > 64 * sizeof(CHAR) )
    {
      ABORT( status, INITSEARCHH_EDFLG, INITSEARCHH_MSGEDFLG ); 
    }

    /* parse simType */
    if ( sscanf( initSearchParams->argv[INSPARG_CRVC], 
          "%u, %u, %u, %[1234567890,]", 
          &(params->inspiralDebugFlag), &(params->seed), 
          &crvcFlag, junkBuffer ) - 4 )
    {
      ABORT( status, INITSEARCHH_EDFLG, INITSEARCHH_MSGEDFLG ); 
    }

    /* read the simCount on the master and slave if this is a bank sim */
    if ( params->inspiralDebugFlag == 5 )
    {
      if ( sscanf( initSearchParams->argv[INSPARG_CRVC], 
            "%*u, %*u, %*u, %u, %[1234567890,]", 
            &(params->simCount), junkBuffer ) - 2 )
      {
        ABORT( status, INITSEARCHH_EDFLG, INITSEARCHH_MSGEDFLG ); 
      }
    }
    else
    {
      params->simCount = 1;
    }

    /* if this is a slave, parse the rest of the arguments */
    if ( ! params->searchMaster ) 
    {
      /* allocate memory for the simulation parameter structure */
      if ( ! (slaveParamsPtr->simParams = (InspiralSimulationParams *) 
            LALCalloc( 1, sizeof(InspiralSimulationParams) )) )
      {
        ABORT( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
      }

      slaveParamsPtr->simParams->simCount = params->simCount;

      /* parse whether or not to write filter output */
      if ( crvcFlag )
      {
        /* need to do this as BOOLEAN is really an unsigned char */
        initParamsPtr->createRhosqVec = 1;
        params->dumpData = 1;
      }

      /* read the rest of the parameters as appropriate to simulation type */
      switch ( params->inspiralDebugFlag )
      {
        case 2:
          slaveParamsPtr->simParams->simType = fcGaussianNoise;

          /* read variance of gaussian noise */
          if ( sscanf( initSearchParams->argv[INSPARG_CRVC], 
                "%*u, %*u, %*u, %f",
                &(slaveParamsPtr->simParams->gaussianVarsq) ) - 1 )
          {
            ABORT( status, INITSEARCHH_EDFLG, INITSEARCHH_MSGEDFLG ); 
          }

          break;

        case 3:
          slaveParamsPtr->simParams->simType = fcGaussianNoiseInject;

          /* read variance of gaussian noise */
          if ( sscanf( initSearchParams->argv[INSPARG_CRVC], 
                "%*u, %*u, %*u, %f",
                &(slaveParamsPtr->simParams->gaussianVarsq) ) - 1 )
          {
            ABORT( status, INITSEARCHH_EDFLG, INITSEARCHH_MSGEDFLG ); 
          }

          break;

        case 4:
          slaveParamsPtr->simParams->simType = fcRealDataInject;

          break;

        case 5:
          slaveParamsPtr->simParams->simType = fcBankMinimalMatch;

          /* read upper and lower masses of random signals to generate */
          if ( sscanf( initSearchParams->argv[INSPARG_CRVC], 
                "%*u, %*u, %*u, %*u, %f, %f, %f",
                &(slaveParamsPtr->simParams->mMin),
                &(slaveParamsPtr->simParams->mMax),
                &(slaveParamsPtr->simParams->fLowInject) ) - 3 )
          {
            ABORT( status, INITSEARCHH_EDFLG, INITSEARCHH_MSGEDFLG ); 
          }

          break;

        default:
          ABORT( status, INITSEARCHH_EDFLG, INITSEARCHH_MSGEDFLG ); 
          break;
      }
    }
  }

  initParamsPtr->ovrlap = params->ovrlap = 
    (UINT4) atoi( initSearchParams->argv[INSPARG_OVRL] );

  initParamsPtr->numPoints = 
    atoi( initSearchParams->argv[INSPARG_NPTS] );
  initParamsPtr->numSegments = 
    atoi( initSearchParams->argv[INSPARG_NSEG] );
  initParamsPtr->numChisqBins = 
    atoi( initSearchParams->argv[INSPARG_NCHI] );


  /* decide wether or not to write out the filter output */
  if ( params->inspiralDebugFlag == 1 )
  {
    initParamsPtr->createRhosqVec = 1;
    params->dumpData = 1;
  }

  params->inputDataLength = 
    initParamsPtr->numPoints * initParamsPtr->numSegments - 
    ( initParamsPtr->numSegments - 1 ) * params->ovrlap;


  /*
   *
   * populate InspiralMasterParams structure
   *
   */


  if ( params->searchMaster )
  {
    masterParamsPtr->numTmpltExch = 
      (UINT4) atoi( initSearchParams->argv[INSPARG_NTXC] );

    masterParamsPtr->inspiralDebugFlagPtr = &(params->inspiralDebugFlag);
  }


  /*
   *
   * populate InspiralSlaveParams structure
   *
   */


  if ( ! params->searchMaster )
  {
    slaveParamsPtr->dataConditioned = 0;

    slaveParamsPtr->inspiralDebugFlagPtr = &(params->inspiralDebugFlag);

    slaveParamsPtr->rhosqThreshVec = 
      (REAL4 *) LALCalloc( 2, sizeof(REAL4) );
    if ( sscanf( initSearchParams->argv[INSPARG_RHOT], "%f, %f\n", 
          slaveParamsPtr->rhosqThreshVec, 
          slaveParamsPtr->rhosqThreshVec + 1 ) - 2 )
    {
      ABORT( status, INITSEARCHH_ERHOT, INITSEARCHH_MSGERHOT ); 
    }

    slaveParamsPtr->chisqThreshVec = 
      (REAL4 *) LALCalloc( 2, sizeof(REAL4) );
    if ( sscanf( initSearchParams->argv[INSPARG_CHIT], "%f, %f\n", 
          slaveParamsPtr->chisqThreshVec, 
          slaveParamsPtr->chisqThreshVec + 1 ) - 2 )
    {
      ABORT( status, INITSEARCHH_ECHIT, INITSEARCHH_MSGECHIT ); 
    }

    params->specType = slaveParamsPtr->specType = (InspiralSpecType) 
      atoi( initSearchParams->argv[INSPARG_SPEC] );

    LALCreateFindChirpSegmentVector( status->statusPtr, 
        &(slaveParamsPtr->fcSegVec), initParamsPtr );
    CHECKSTATUSPTR( status );

    LALFindChirpSPDataInit( status->statusPtr, 
        &(slaveParamsPtr->dataParams), initParamsPtr );
    CHECKSTATUSPTR( status );
    slaveParamsPtr->dataParams->invSpecTrunc = 
      atoi( initSearchParams->argv[INSPARG_TRNC] );
    slaveParamsPtr->dataParams->fLow = 
      (REAL4) atof( initSearchParams->argv[INSPARG_FLOW] );
    slaveParamsPtr->dataParams->dynRange = 
      pow( 2.0, (REAL4) atof( initSearchParams->argv[INSPARG_DYNR] ) );

    LALFindChirpSPTemplateInit( status->statusPtr, 
        &(slaveParamsPtr->tmpltParams), initParamsPtr );
    CHECKSTATUSPTR( status );
    slaveParamsPtr->tmpltParams->fLow = 
      (REAL4) atof( initSearchParams->argv[INSPARG_FLOW] );
    slaveParamsPtr->tmpltParams->dynRange = 
      pow( 2.0, (REAL4) atof( initSearchParams->argv[INSPARG_DYNR] ) );

    LALFindChirpFilterInit( status->statusPtr, 
        &(slaveParamsPtr->filterParams), initParamsPtr );
    CHECKSTATUSPTR( status );
    slaveParamsPtr->filterParams->computeNegFreq = 0;
    slaveParamsPtr->filterParams->maximiseOverChirp = 
      (UINT4) atoi( initSearchParams->argv[INSPARG_MAXC] );

    LALCreateFindChirpInput( status->statusPtr, 
        &(slaveParamsPtr->filterInput), initParamsPtr );
    CHECKSTATUSPTR( status );
    LALFindChirpChisqVetoInit( status->statusPtr, 
        slaveParamsPtr->filterParams->chisqParams, 
        initParamsPtr->numChisqBins,
        initParamsPtr->numPoints );
    CHECKSTATUSPTR( status );

    slaveParamsPtr->useMPI = 1;

    slaveParamsPtr->mpiComm = NULL;
  }



  /*
   *
   * search master: templates from bank generation routines
   *
   */


  if ( params->searchMaster && params->tmpltSource == 0 )
  {
    ABORT( status, INITSEARCHH_EBANK, INITSEARCHH_MSGEBANK );
  }


  /*
   *
   * search master: templates from command line
   *
   */


  if ( params->searchMaster && params->tmpltSource == 1 )
  {
    InspiralTmpltsFromCmdLine( status->statusPtr, 
        params, initSearchParams->argv );
    CHECKSTATUSPTR( status );
  }


 /*
  *
  * allocate memory for data segments in parameter structure
  *
  */


  if ( ! params->searchMaster )
  {
    /* clear the time and frequency series which store the input data   */
    memset( &(params->chan), sizeof(REAL4TimeSeries), 0 );
    memset( &(params->spec), sizeof(REAL4FrequencySeries), 0 );
    memset( &(params->resp), sizeof(COMPLEX8FrequencySeries), 0 );

    /* if this is a simulation, store pointer to the input data */
    if ( slaveParamsPtr->simParams )
    {
      slaveParamsPtr->simParams->chan = &(params->chan);
    }

    /* allocate memory for the time series input data */
    if ( ! (params->chan.data = (REAL4Sequence *) 
          LALCalloc( 1, sizeof(REAL4Sequence) )) )
    {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }

    /* allocate memory for the frequency sequences in the spec and resp */
    if ( ! (params->spec.data = (REAL4Sequence *) 
        LALCalloc( 1, sizeof(REAL4Sequence) )) )
    {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }

    if ( ! (params->resp.data = (COMPLEX8Sequence *) 
        LALCalloc( 1, sizeof(COMPLEX8Sequence) )) )
    {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }
  }


  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}


#ifdef INSPIRAL_SO_GRAPHING_FUNCS
/* --- function to graph an array of REAL4 ------------------------------ */
void
graphREAL4 (
    REAL4      *array, 
    INT4        n,
    INT4        spacing
           ) 
{
  FILE *fp;
  INT4 i;

  /* open a file for writing */
  if ( !(fp = fopen( "temp.graph", "w" )) )
  {
    printf( "couldn't open file\n" );
  }

  /* print data into the file */
  for ( i = 0; i < n; ++i )
    fprintf( fp, "%d\t%e\n", i, array[i * spacing] );

  /* close the file */
  fclose( fp );

  return;
}
#endif
