/***************************** <lalVerbatim file="StochasticApplySearchCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: SGWBApplySearch.c,v 1.7 2003/10/07 17:17:09 jolien Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticApplySearchC">
\subsection{Module \texttt{StochasticApplySearch.c}}
\label{s:StochasticApplySearch.c}

Routine that implements the stochastic background search, calculating
the values of the spectrum of the optimally-filtered cross-correlation 
statistic, building the output for subsequent insertion into the 
metadata database and frames.

\subsubsection*{Prototypes}
\input{StochasticApplySearchCP}
\index{\texttt{LALApplySearch()}}

\subsection*{Description}

Implements the search for a stochastic background, calculating 
the values of the spectrum of the optimally-filtered cross-correlation 
statistic with $\Omega_{\rm gw}(f) = \Omega_0 = {\rm const}$.
The cross-correlation statistic spectrum values are then built for 
subsequent insertion into the metadata database and frames using the 
\texttt{StochasticBuildOutput} routine.

\subsection*{Algorithms}

Given time-series data from two interferometers (pre-processed by
the \texttt{datacondAPI}) and values of the optimal filter 
(calculated in \texttt{LALConditionData}), \texttt{LALApplySearch}
does the following:
%
\begin{enumerate}
\item 
Splits the input time-series data into \texttt{numSegments$\ge 10$}
segments, each of length \texttt{numPoints}.

\item 
Windows, zero-pads, and FFTs each segment of time-series data.

\item 
Calculates the values of the standard optimally-filtered 
cross-correlation statistic for each segment of data, using the 
values of the optimal filter previously calculated in 
\texttt{LALConditionData}.

\item
Coarse-grains the cross-correlation statistic spectrum output to 
the desired bandwidth and frequency resolution.

\item
Builds the cross-correlation statistic spectrum output 
(for each data segment) for subsequent insertion into the metadata 
database and frames.
\end{enumerate}

\subsection*{Uses}
 
\begin{itemize}
\item \texttt{LALCCreateVector}
\item \texttt{LALSZeroPadAndFFT}
\item \texttt{LALStochasticCrossCorrelationSpectrum}
\item \texttt{LALCCoarseGrainFrequencySeries}
\item \texttt{StochasticBuildOutput} 
\item \texttt{LALCDestroyVector}
\end{itemize}

\subsection*{Notes}

The analysis is run on {\em only} the search master node of the 
beowulf, 
since we are currently considering the simple case of stochastic 
signals with $\Omega_{\rm gw}(f)=\Omega_0 = {\rm const}$.
In the future, searches for stochastic backgrounds having different 
spectra (e.g., power law $\Omega_{\rm gw}(f)\propto f^\alpha$) 
should be run on multiple slaves (e.g., one node for each $\alpha$).

\vfill{\footnotesize\input{StochasticApplySearchCV}}

*************************************************** </lalLaTeX> */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/TimeFreqFFT.h>
#include <lal/CoarseGrainFrequencySeries.h>
#include "SGWBApplySearch.h"
#include "SGWB.h"
#include "CastToFunctions.h"
#include "SGWBAuxFunctions.h"

NRCSID (SGWBAPPLYSEARCHC, "$Id: SGWBApplySearch.c,v 1.7 2003/10/07 17:17:09 jolien Exp $");

/************************ <lalVerbatim file="StochasticApplySearchCP"> */
void LALApplySearch(
		    LALStatus             *status,
		    LALSearchOutput       *output,
		    LALSearchInput        *input,
		    LALApplySearchParams  *applyParams
		    )
     /*********************************************** </lalVerbatim> */
{

  INT4           myRank;
  MPIMessage     myMpiMsg; /* initialization message */
  MPI_Comm       myMpiComm; 
  INT4           nodeNumSegs;

  StochasticSearchParams*             params = NULL;

  INITSTATUS (status, "LALApplySearch", SGWBAPPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

  /* 
   *
   * check output and parameters 
   * 
   */

  ASSERT (output, 
	  status, 
	  STOCHASTICAPPLYSEARCHH_ENULL, 
	  STOCHASTICAPPLYSEARCHH_MSGENULL);

  ASSERT (!(output->result), 
	  status, 
	  STOCHASTICAPPLYSEARCHH_ENNUL, 
	  STOCHASTICAPPLYSEARCHH_MSGENNUL);

  ASSERT (applyParams, 
	  status, 
	  STOCHASTICAPPLYSEARCHH_ENULL, 
	  STOCHASTICAPPLYSEARCHH_MSGENULL);

  ASSERT (applyParams->mpiParams, 
	  status, 
	  STOCHASTICAPPLYSEARCHH_ENULL, 
	  STOCHASTICAPPLYSEARCHH_MSGENULL);

  ASSERT (applyParams->searchParams, 
	  status, 
	  STOCHASTICAPPLYSEARCHH_ENULL, 
	  STOCHASTICAPPLYSEARCHH_MSGENULL);


  /* cast the parameter structure */ 

  params = (StochasticSearchParams *) applyParams->searchParams;


  /* 
   *
   * perform the search
   *
   */

  nodeNumSegs = params->initParams->nodeNumSegs;

  /******************************************************
	     
	  MPI Initialization for all nodes

  ******************************************************/

  if ( !params->numSlaves )
    {
      /* allocate memory and check for success   */

      fprintf(stdout," not numSlaves \n ");
      fflush( stdout );

      params->numSlaves = (UINT4 *) LALMalloc( sizeof(UINT4) );
      memset( params->numSlaves, 0, sizeof(UINT4) );
    
      if( !params->numSlaves )
	{
	  ABORT( status, 
		 STOCHASTICAPPLYSEARCHH_EALOC, STOCHASTICAPPLYSEARCHH_MSGEALOC );
	}
    
      MPI_Comm_size( *(applyParams->mpiParams->comm), params->numSlaves );
      *(params->numSlaves) -= 1;  /*  do it only once for the first call */
      myMpiMsg.send=1;

      fprintf(stdout," numSlaves %i \n ", *(params->numSlaves));
      fflush( stdout );
    }/* endif firstTime */

  /* get the number of search slaves and my Rank (0 for master) */

  MPI_Comm_rank( *(applyParams->mpiParams->comm), &myRank );
  myMpiComm   = *(applyParams->mpiParams->comm);
  myMpiMsg.source = myRank;
  output->numOutput = 0;
  
#ifdef STOCHASTIC_SO_DEBUG
  /*   debug */
  fprintf(stdout,"I am node number %i,",myRank);
  fflush( stdout );
#endif



  /*****************************************************
   *****************************************************   
                          MPI Master activity 
			  ***************************************		      
			  ***************************************/	

  if ( params->searchMaster )
    {
#ifdef STOCHASTIC_SO_DEBUG
      fprintf(stdout," the ***master***, lazy and always a bottleneck!  \n ");
      fflush( stdout );
#endif
      if ( *(params->numSlaves) )
	{

	  /* #ifdef STOCHASTIC_SO_DEBUG */
	  fprintf(stdout,"master of rank %i  waiting for slave's msgs... %d\n",myRank, *(params->numSlaves));
	  fflush( stdout );
	  /* #endif */

	  /* wait for a message */
	  LALMPIRecvMsg (status->statusPtr, &myMpiMsg, myMpiComm);
	  CHECKSTATUSPTR (status);

#ifdef STOCHASTIC_SO_DEBUG
	  fprintf(stdout,"master of rank %i received the %i th  slave's msgs...\n", myRank,params->currentSegment);
	  fflush( stdout );
#endif

	  switch (myMpiMsg.msg)
	    {
	
	    case RoutineExch: 
	      params->masterCounter++;
/* 	      output->fracRemaining = 1.0 - (REAL4) (params->masterCounter)/(params->initParams->numSegments); */
	      output->fracRemaining = 0.34;
	      output->notFinished   = STOCHASTICH_TRUE;
	
#ifdef STOCHASTIC_SO_DEBUG
	      fprintf(stdout,"for node %i, the master,  numSlave, segcom,                 and fracrem= %i,%i,%g\n",myRank,*(params->numSlaves), params->currentSegment,output->fracRemaining ); 
	      fflush( stdout );
#endif

	      break;

	    case ExchFinished:
	      params->masterCounter++;
/* 	      output->fracRemaining = 1.0 - (REAL4) (params->masterCounter)/(params->initParams->numSegments) - 0.2; */
	      output->fracRemaining = 0.34;
	      --*(params->numSlaves);
	      break;
       
	      /* unknown type then quit  */
	    default: 
	      ABORT( status, STOCHASTICDATAH_EUEXT, STOCHASTICDATAH_MSGEUEXT );
	
	    }/* end switch */
	}  /* endif params->numSlave !=0 */
      else /*  params->numSlaves=0 */
	{

#ifdef STOCHASTIC_SO_DEBUG
	  fprintf(stdout,"master of node %i is out of job ! \n",myRank);
	  fflush( stdout );
#endif

	  output->fracRemaining = 0;
	  output->notFinished   = 0;
	}
    }/*  endif searchMaster */
  /***********************************************************
   **************************************************
                      Slave Activity	
		      *************************************************	
		      ***************************************************/ 
  else  
    { /* if I am a slave */
#ifdef FCT_SO_DEBUG
    
      fprintf(stdout,"the diligent **slave**, hard working and underpaid \n ");
      fflush( stdout );
#endif

      printf("\n currentseg %d\n", params->currentSegment);
      
      params->currentSegment = ((myRank - 1) * nodeNumSegs) + params->slaveCounter;
           
      printf("\n *** 105 *** slaveCounter1 %d currentSegment %d *** \n", params->slaveCounter, params->currentSegment);
      
      if (params->currentSegment >= 105) {
	printf("\nlkdslkfeiol***************kldfjlkdsaklfdl*****\n");
	printf("**** the diligent **slave**, hard working and underpaid %d slavecount %d nodenumsegs %d currentseg %d \n", myRank, params->slaveCounter, nodeNumSegs, params->currentSegment); 
      }

      if (params->currentSegment < params->initParams->numSegments
	  && (params->currentSegment < ((myRank) * nodeNumSegs)
	      && params->currentSegment >=  ((myRank - 1) * nodeNumSegs)))
	{
	  TRY( LALSGWBCoreApplySearch (status->statusPtr,
				       output,
				       input,
				       params),
	       status);

	  /*
	   *  
	   * compute fraction of search remaining for wrapper and set 
	   * output not finished 
	   * 
	   * :NOTE: this is what duncan did for simple.so even though
	   * it would seem that i should increment to the pointer
	   * to the current segment before computing fracRemaining and
	   * before determining how to set notFinished.
	   *
	   */  
     
	  /* 	  output->fracRemaining = 1.0 -  */
	  /* 	    (REAL4)(params->slaveCounter)/(REAL4)(nodeNumSegs); */
	  output->fracRemaining = 0.34;
	  output->notFinished = STOCHASTICH_TRUE;
	  myMpiMsg.msg = RoutineExch;
	 
	  printf("\n slaveCounter1 %d\n", params->slaveCounter);
	  /* increment pointer to current segment */
	  (params->slaveCounter)++;
	  printf("\n slaveCounter2 %d\n", params->slaveCounter);

	} /* end if over segments */
      else
	{
	  /* we are finished with the search */
	  /* printf("\n\n *************** Hey We Finished  ************** \n\n"); */
	  output->fracRemaining = 0;
	  output->notFinished   = STOCHASTICH_FALSE;
	  myMpiMsg.msg = ExchFinished;	
	}

      LALMPISendMsg (status->statusPtr, &myMpiMsg, 0,myMpiComm);
      CHECKSTATUSPTR (status);    
    }

  /* normal exit */ 
  DETATCHSTATUSPTR (status);
  RETURN (status);
}

  
  
  
  
