/********************** <lalVerbatim file="StochasticBuildOutputCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: SGWBBuildOutput.c,v 1.3 2003/09/03 02:37:17 kaice Exp $
 **************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticBuildOutputC">
\subsection{Module \texttt{StochasticBuildOutput.c}}
\label{s:StochasticBuildOutput.c}

Routine that builds the output of the optimally-filtered
cross-correlation statistic spectrum for insertion into the metadata 
database and frames.

\subsubsection*{Prototypes}
\input{StochasticBuildOutputCP}
\index{\texttt{StochasticBuildOutput()}}
\index{\texttt{LALCreateSearchSummvarsDatabase()}}
\index{\texttt{LALCreateDBEntryBLOBU()}}

\subsection*{Description}

\texttt{StochasticBuildOutput()}
builds the output of the optimally-filtered cross-correlation
statistic spectrum for subsequent insertion into the metadata
database (as a \texttt{summ\_spectrum} table) and frames (as a
sequence in \texttt{filter\_output}).

\texttt{LALCreateSearchSummvarsDatabase()} creates a
\texttt{search\_summvars} database entry, analogous to
\texttt{LALCreateSearchSummaryDatabase()}, while
\index{\texttt{LALCreateDBEntryBLOBU()}} is a version of
\index{\texttt{LALCreateDBEntryBLOB()}} which treats the BLOB as
\texttt{(unsigned char *)} rather than \texttt{(signed char *)}.

\subsection*{Algorithms}

\subsection*{Uses}
 
\begin{itemize}
\item \texttt{LALCreateSearchSummaryDataBase}
\item \texttt{LALCreateSearchSummvarsDataBase}
\item \texttt{LALCHARCreateVector}
\item \texttt{UnitAsString}
\item \texttt{LALCHARDestroyVector}
\item \texttt{LALCreateNextDBEntry}
\item \texttt{LALCreateDBEntryData}
\item \texttt{LALCreateDBEntryBLOB}
\item \texttt{LALCreateDBEntryBLOBU}
\item \texttt{LALMalloc}
\item \texttt{LALCalloc}
\end{itemize}

\subsection*{Notes}

\vfill{\footnotesize\input{StochasticBuildOutputCV}}

*************************************************** </lalLaTeX> */



#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/Units.h>
#include "SGWBApplySearch.h"

NRCSID (SGWBBUILDOUTPUTC, "power $Id: SGWBBuildOutput.c,v 1.3 2003/09/03 02:37:17 kaice Exp $");

/************************ <lalVerbatim file="StochasticBuildOutputCP"> */
void StochasticBuildOutput (
      LALStatus*                  status,
      LALSearchOutput*            output,
      StochasticSearchOutput*     input, 
      StochasticSearchParams     *params
)
/*********************************************** </lalVerbatim> */
{
   outPut                 *outputPtr;
   dataBase               *dbEntry;
   LALBLOB                  blob;
   SearchSummaryInfo        searchSummInfo;
   SearchSummaryVariables   searchSummVars;

   UINT8           segLengthNanoSeconds;
   UINT8           segStartNanoSeconds;
   UINT8           segStopNanoSeconds;
   INT4            stopSec;
   INT4            stopNan;
   UINT4           i;
   multiDimData*   mddEntry;
   CHARVector     *unitString;
   CHAR            string[STOCHASTICAPPLYSEARCHH_MAXSTRINGLENGTH];
#ifdef BUILDBH_LDAS_0_6
   REAL8           outputValue;
#endif

   INITSTATUS (status, "SGWBBuildOutput", SGWBBUILDOUTPUTC);
   ATTATCHSTATUSPTR (status);

#ifdef BUILDBH_LDAS_0_6
   searchSummVars.value = &outputValue;
#endif

   output->numOutput = 0;

   if ( input->ccStatReal )
   {
     ++output->numOutput;
   } else if ( input->ccStatComplex )
   {
     output->numOutput += 2;   /* real and imaginary parts */
   }
   if ( input->sigma )
   {
     ++output->numOutput;
   }
   if ( input->ccSpec )
   {
     ++output->numOutput;
   }

   if ( (params->currentSegment == 0) 
	&& ( input->ccStatReal || input->sigma || input->ccStatComplex) )
   {
     output->numOutput += 2; /* info plus segment length */
   }

   if ( !(output->numOutput) )
   {
     DETATCHSTATUSPTR (status);
     RETURN (status);
   }

   /* 
    *
    * create memory for the outPut array
    *
    */

   /* using LALCalloc protects us against new members of the structure */
   outputPtr = output->result 
     = (outPut *) LALCalloc (output->numOutput, sizeof (outPut));
   if (!outputPtr)
   {
      ABORT (status, 
	     STOCHASTICAPPLYSEARCHH_EALOC, 
	     STOCHASTICAPPLYSEARCHH_MSGEALOC);
   }

   /*
    *
    * fill the array of output
    *
    */

   if ( (params->currentSegment == 0) 
	&& ( input->ccStatReal || input->sigma || input->ccStatComplex) )
   {
     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     /* Write the search summary to the database */

     searchSummInfo.dsoName     = STOCHASTICAPPLYSEARCHH_DSONAME;
     searchSummInfo.inputStartTime.gpsSeconds 
                             = params->timeInterval.startSec;
     searchSummInfo.inputStartTime.gpsNanoSeconds 
                             = params->timeInterval.startNan;
     searchSummInfo.inputEndTime.gpsSeconds 
                             = params->timeInterval.stopSec;
     searchSummInfo.inputEndTime.gpsNanoSeconds 
                             = params->timeInterval.stopNan;
     searchSummInfo.outputStartTime.gpsSeconds 
                             = params->timeInterval.startSec;
     searchSummInfo.outputStartTime.gpsNanoSeconds 
                             = params->timeInterval.startNan;
     searchSummInfo.outputEndTime.gpsSeconds 
                             = params->timeInterval.stopSec;
     searchSummInfo.outputEndTime.gpsNanoSeconds 
                             = params->timeInterval.stopNan;
     /* The concept of an "event" is ill-defined for stochastic searches */
     searchSummInfo.numEvents = 0;
     /* :TODO: change this if we start using search slaves */
     searchSummInfo.numNodes = *(params->numSlaves) + 2;
     
     dbEntry = NULL;
     TRY( LALCreateSearchSummaryDataBase( status->statusPtr, &dbEntry,
					  &searchSummInfo),
	  status );

     outputPtr->results = dbEntry;
     
     ++outputPtr;

     /* Now write the length of a segment associated with this search job */

     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     /* Write the length of a segment to the database */

     searchSummVars.name   = STOCHASTICAPPLYSEARCHH_SEGLENNAME;
#ifdef BUILDBH_LDAS_0_6
     outputValue = (REAL8) params->segmentDuration;
#else
     searchSummVars.value  = &(params->segmentDuration);
#endif
     /* Put units in the string field */
     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	  status );
     TRY( LALUnitAsString(status->statusPtr, unitString, &lalSecondUnit),
	  status );

     searchSummVars.string = unitString->data;
     /* strncpy( searchSummVars.string, 
	unitString->data, strlen(unitString->data)*sizeof(CHAR) );  */

     dbEntry = NULL;
     TRY( LALCreateSearchSummvarsDataBase( status->statusPtr, &dbEntry,
					  &searchSummVars),
	  status );

     TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	  status );

     outputPtr->results = dbEntry;
     
     ++outputPtr;
   }

   if (input->ccSpec)
   {
     /* Find end time of segment */
     segStartNanoSeconds = 1000000000L 
       * (INT8) input->ccSpec->epoch.gpsSeconds;
     segStartNanoSeconds += (INT8) input->ccSpec->epoch.gpsNanoSeconds;
     segLengthNanoSeconds = (INT8) 
       (1.0e9 * (UINT8) (params->initParams->numPoints)
	* params->timeInterval.timeStepSize);
     segStopNanoSeconds = segStartNanoSeconds + segLengthNanoSeconds;
     stopSec = (INT4) (segStopNanoSeconds / 1000000000L);
     stopNan = (INT4) (segStopNanoSeconds % 1000000000L);

     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     if ( STOCHASTICH_CCSPECDBBIT & input->outputType )
     {
       
       /* Write the CC spectrum to the database */

       dbEntry = NULL;
       
       /* start time seconds */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "start_time"),
	    status );
       outputPtr->results = dbEntry;
       TRY( LALCreateDBEntryData( status->statusPtr, dbEntry, int_4s, 1),
	    status );
       dbEntry->rows.int4s[0] = (INT4) input->ccSpec->epoch.gpsSeconds;
       
       /* start time nanoseconds */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "start_time_ns"),
	    status );
       TRY( LALCreateDBEntryData( status->statusPtr, dbEntry, int_4s, 1),
	    status );
       dbEntry->rows.int4s[0] =	(INT4) input->ccSpec->epoch.gpsNanoSeconds;

       /* end time seconds */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "end_time"),
	    status );
       TRY( LALCreateDBEntryData( status->statusPtr, dbEntry, int_4s, 1),
	    status );
       dbEntry->rows.int4s[0] = stopSec;
       
       /* end time nanoseconds */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "end_time_ns"),
	    status );
       TRY( LALCreateDBEntryData( status->statusPtr, dbEntry, int_4s, 1),
	    status );
       dbEntry->rows.int4s[0] = stopNan;

       /* frames used (?) */
       /* This information is not available to the DSO, so we set it to -1 */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "frames_used"),
	    status );
       TRY( LALCreateDBEntryData( status->statusPtr, dbEntry, int_4s, 1),
	    status );
       dbEntry->rows.int4s[0] = -1; 
      
       /* start frequency */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "start_frequency"),
	    status );
       TRY( LALCreateDBEntryData( status->statusPtr, dbEntry, real_8, 1),
	    status );
       dbEntry->rows.real8[0] = input->ccSpec->f0;
       
       /* frequency spacing */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "delta_frequency"),
	    status );
       TRY( LALCreateDBEntryData( status->statusPtr, dbEntry, real_8, 1),
	    status );
       dbEntry->rows.real8[0] = input->ccSpec->deltaF;

       /* Key for program name. */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "program"),
	    status );
       blob.data = STOCHASTICAPPLYSEARCHH_PROGNAME;
       blob.size = strlen(blob.data);
       TRY( LALCreateDBEntryBLOB( status->statusPtr, dbEntry, &blob, 1 ),
	    status );

       /* Key for spectrum type. */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "mimetype"),
	    status );
       blob.data = STOCHASTICAPPLYSEARCHH_MIMETYPE;
       blob.size = strlen(blob.data);
       TRY( LALCreateDBEntryBLOB( status->statusPtr, dbEntry, &blob, 1 ),
	    status );

       /* key for channel name */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "channel"),
	    status );
       /* TODO: Is this right??? */
       blob.data = STOCHASTICAPPLYSEARCHH_CHANNEL;
       blob.size = strlen(blob.data);
       TRY( LALCreateDBEntryBLOB( status->statusPtr, dbEntry, &blob, 1 ),
	    status );

       /* Key for spectrum type. */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "spectrum_type"),
	    status );
       blob.data = STOCHASTICAPPLYSEARCHH_SPECTYPE;
       blob.size = strlen(blob.data);
       TRY( LALCreateDBEntryBLOB( status->statusPtr, dbEntry, &blob, 1 ),
	    status );

       /* spectrum */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "spectrum"),
	    status );
       /* Spectrum must be stored as a BLOb */
       /* This is ugly, but should work if there's no padding in storage
	* of COMPLEX8*  */
       blob.data = input->ccSpec->data->data;
       blob.size = input->ccSpec->data->length 
	 * (sizeof(COMPLEX8) / sizeof(UCHAR)) ;
       TRY( LALCreateDBEntryBLOBU( status->statusPtr, dbEntry, &blob, 1 ),
	    status );

       /* spectrum_length */
       TRY( LALCreateNextDBEntry( status->statusPtr, &dbEntry,
				  "summ_spectrum", "spectrum_length"),
	 status );
       TRY( LALCreateDBEntryData( status->statusPtr, dbEntry, int_4s, 1),
	    status );
       dbEntry->rows.int4s[0] = input->ccSpec->data->length;
     }

     if ( STOCHASTICH_CCSPECFRBIT & input->outputType )
     {
       /* 
	*
	* alloc memory for the optional linked list
	*
	*/
       
       
       /* allocate memory */
       /* using LALCalloc protects us against new members of the structure */
       mddEntry = outputPtr->optional = (multiDimData *) 
	 LALCalloc (1, sizeof (multiDimData));
       if (!mddEntry)
       {
	 ABORT (status, 
		STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC);
       }
       
       /* write the name and domain of the data */
       strncpy (mddEntry->name, "filter_output", maxMultiDimName);
       
       /* Read the dimensions from the output frequency series */
       
       unitString = NULL;
       TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	    status );
       TRY( LALUnitAsString(status->statusPtr, unitString, &(input->ccSpec->sampleUnits)),
	    status );
       strncpy (mddEntry->units, unitString->data, maxMultiDimUnits);
       TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	    status );
       mddEntry->space = freqD;
       
       mddEntry->range.dFreq.numberSamples = input->ccSpec->data->length;
       
       
       mddEntry->range.dFreq.gpsStartTimeSec 
	 = input->ccSpec->epoch.gpsSeconds;
       mddEntry->range.dFreq.gpsStartTimeNan 
	 = input->ccSpec->epoch.gpsNanoSeconds;
       mddEntry->range.dFreq.gpsStopTimeSec = stopSec;
       mddEntry->range.dFreq.gpsStopTimeNan = stopNan;
       /* The start and stop frequencies are relative to the base
	  frequency per the v6 frame spec */
       mddEntry->range.dFreq.startFreq = input->ccSpec->f0 
	 - params->initParams->baseFreq;
       mddEntry->range.dFreq.stopFreq 
	 = input->ccSpec->f0 
	 + (REAL8)(input->ccSpec->data->length - 1) * input->ccSpec->deltaF
	 - params->initParams->baseFreq;
       mddEntry->range.dFreq.freqStepSize = input->ccSpec->deltaF;
       mddEntry->range.dFreq.baseFreq = params->initParams->baseFreq;       
       
       /* data type is COMPLEX8 */
       mddEntry->type = complex_8;
       
       /* size of the data */
       mddEntry->numberDimensions = 1;
       mddEntry->dimensions = (UINT4 *) LALMalloc (sizeof(UINT4));
       if (!(mddEntry->dimensions))
       {
	 ABORT (status, 
		STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
       mddEntry->dimensions[0] = input->ccSpec->data->length;
       
       /* dcHistory is meaningless here */
       mddEntry->history = NULL;
       
       /* allocate memory for the data */
       mddEntry->data.complex8 = (COMPLEX8 *) 
	 LALCalloc (input->ccSpec->data->length, sizeof(COMPLEX8));
       if (!(mddEntry->data.complex8))
       {
	 ABORT (status, 
		STOCHASTICAPPLYSEARCHH_EALOC, 
		STOCHASTICAPPLYSEARCHH_MSGEALOC );
       }
   
       /* and copy the data there */
       memcpy (mddEntry->data.complex8, 
	       input->ccSpec->data->data, 
	       input->ccSpec->data->length * sizeof(COMPLEX8) );
     }
     ++outputPtr;
   }
   if ( input->sigma )
   {
     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     /* Write sigma to the database */

     sprintf(string, STOCHASTICAPPLYSEARCHH_SIGMANAME,
	     params->currentSegment);
     searchSummVars.name   = string;
#ifdef BUILDBH_LDAS_0_6
     outputValue = (REAL8) input->sigma->value;
#else
     searchSummVars.value  = &(input->sigma->value);
#endif
     /* Put units in the string field */
     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	  status );
     TRY( LALUnitAsString(status->statusPtr, unitString, &(input->sigma->units)),
	  status );

     searchSummVars.string = unitString->data;
     /* strncpy( searchSummVars.string, 
	unitString->data, strlen(unitString->data)*sizeof(CHAR) ); */

     dbEntry = NULL;
     TRY( LALCreateSearchSummvarsDataBase( status->statusPtr, &dbEntry,
					  &searchSummVars),
	  status );

     TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	  status );

     outputPtr->results = dbEntry;
     
     ++outputPtr;
   }
   if ( input->ccStatReal )
   {
     /* this is the same for every output */
     outputPtr->indexNumber = 0;
     outputPtr->search         = stocastic;
     /* :TODO: note the incorrect spelling above.  it was entered */
     /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
     /* that there is another misspelling in that same file:       */
     /* catagory should spelled category */
     outputPtr->significant    = STOCHASTICH_TRUE;
     outputPtr->states         = NULL;
     outputPtr->optional       = NULL;
     
     /* Write the CC statistic to the database */
     
     sprintf(string, STOCHASTICAPPLYSEARCHH_CCSTATREALNAME,
	     params->currentSegment);
     searchSummVars.name   = string;
#ifdef BUILDBH_LDAS_0_6
     outputValue = (REAL8) input->ccStatReal->value;
#else
     searchSummVars.value  = &(input->ccStatReal->value);
#endif
     /* Put units in the string field */
     unitString = NULL;
     TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	  status );
     TRY( LALUnitAsString(status->statusPtr, unitString,
			  &(input->ccStatReal->units)),
	  status );
     
     searchSummVars.string = unitString->data;

     dbEntry = NULL;
     TRY( LALCreateSearchSummvarsDataBase( status->statusPtr, &dbEntry,
					  &searchSummVars),
	  status );

     TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	  status );

     outputPtr->results = dbEntry;
     
     ++outputPtr;
   }
   else if ( input->ccStatComplex )
   {
     for ( i=0; i<2; ++i ) {
       /* this is the same for every output */
       outputPtr->indexNumber = 0;
       outputPtr->search         = stocastic;
       /* :TODO: note the incorrect spelling above.  it was entered */
       /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
       /* that there is another misspelling in that same file:       */
       /* catagory should spelled category */
       outputPtr->significant    = STOCHASTICH_TRUE;
       outputPtr->states         = NULL;
       outputPtr->optional       = NULL;
       
       /* Write the CC statistic to the database */
       
       sprintf(string, ( i ? STOCHASTICAPPLYSEARCHH_CCSTATIMAGPARTNAME 
			 : STOCHASTICAPPLYSEARCHH_CCSTATREALPARTNAME ),
	       params->currentSegment);
       searchSummVars.name   = string;
#ifdef BUILDBH_LDAS_0_6
       outputValue = (REAL8) ( i ? 
					   (input->ccStatComplex->value.im) :
					   (input->ccStatComplex->value.re) );
#else
       searchSummVars.value  =  ( i ? 
				  &(input->ccStatComplex->value.im) :
				  &(input->ccStatComplex->value.re) );
#endif

       /* Put units in the string field */
       unitString = NULL;
       TRY( LALCHARCreateVector(status->statusPtr, &unitString,
				LALUnitTextSize),
	    status );
       TRY( LALUnitAsString(status->statusPtr, unitString,
			    &(input->ccStatComplex->units)),
	    status );

       searchSummVars.string = unitString->data;

       dbEntry = NULL;
       TRY( LALCreateSearchSummvarsDataBase( status->statusPtr, &dbEntry,
					     &searchSummVars),
	    status );
       
       TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	    status );

       outputPtr->results = dbEntry;
     
       ++outputPtr;
     }
   }

   /* normal exit */
   DETATCHSTATUSPTR (status);
   RETURN (status);
}

/* <lalVerbatim file="StochasticBuildOutputCP"> */
void
LALCreateDBEntryBLOBU(
    LALStatus *status,
    dataBase  *output,
    LALBLOB   *blob,
    UINT4      rows
    )
{ /* </lalVerbatim> */
  UINT4 row;
  INITSTATUS( status, "LALCreateDBEntryBLOBU", SGWBBUILDOUTPUTC );
  ASSERT( output, status, STOCHASTICAPPLYSEARCHH_ENULL, STOCHASTICAPPLYSEARCHH_MSGENULL );
  ASSERT( rows, status, STOCHASTICAPPLYSEARCHH_EROWS, STOCHASTICAPPLYSEARCHH_MSGEROWS );
  ASSERT( blob, status, STOCHASTICAPPLYSEARCHH_ENULL, STOCHASTICAPPLYSEARCHH_MSGENULL );
  ASSERT( blob->data, status, STOCHASTICAPPLYSEARCHH_ENULL, STOCHASTICAPPLYSEARCHH_MSGENULL );
  ASSERT( blob->size, status, STOCHASTICAPPLYSEARCHH_ESIZE, STOCHASTICAPPLYSEARCHH_MSGESIZE );
  output->type          = char_u_ptr;
  output->numberRows    = rows;
  output->rowDimensions = LALCalloc( rows, sizeof( *output->rowDimensions ) );
  if ( ! output->rowDimensions )
  {
    ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, STOCHASTICAPPLYSEARCHH_MSGEALOC );
  }
  output->rows.chars = LALCalloc( rows, blob->size );
  if ( ! output->rows.chars )
  {
    LALFree( output->rowDimensions );
    output->rowDimensions = NULL;
    ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, STOCHASTICAPPLYSEARCHH_MSGEALOC );
  }
  for ( row = 0; row < rows; ++row )
  {
    output->rowDimensions[row] = blob->size;
    memcpy( output->rows.chars + row * blob->size, blob->data, blob->size );
  }
  RETURN( status );
}
