/************************** <lalVerbatim file="StochasticConditionDataCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: SGWBConditionData.c,v 1.7 2003/09/29 20:11:29 kaice Exp $
**************************************************** </lalVerbatim> */

/**************************** <lalLaTeX file="StochasticConditionDataC">
\subsection{Module \texttt{StochasticConditionData.c}}
\label{s:StochasticConditionData.c}

Routine for performing one-time processing in preparation for the
stochastic search.

\subsubsection*{Prototypes}
\input{StochasticConditionDataCP}
\index{\texttt{LALConditionData()}}

\subsection*{Description}

Performs one-time processing (e.g., extraction of input data, construction
of windowing functions, and
calculation of the optimal filter for a stochastic background having 
$\Omega_{\rm gw}(f)=\Omega_0={\rm const}$) in preparation for the 
stochastic search.

\subsection*{Algorithms}
  
\texttt{LALConditionData} does the following:
%
\begin{enumerate}
\item Constructs the windowing functions requested for the two data streams
\item Checks the validity of the input data, and extracts the 
detector geometry, time-series data, power spectral densities, and 
instrument response functions from the multidim input data.
\item Converts 2-sided power spectral densities (produced by the 
\texttt{datacondAPI}) to 1-sided power spectra having the appropriate 
bandwidth and frequency resolution for construction of the optimal 
filter.
\item Coarse-grains the instrument response functions to the
appropriate bandwidth and frequency resolution for construction of
the optimal filter.
\item Casts the extracted input data to types expected by the LAL 
stochastic routines.
\item Calculates the overlap reduction function given the 
detector geometry information.
\item Sets $\Omega_{\rm gw}(f)=\Omega_0 = {\rm const}$ for this
particular search.
\item Calculates inverse calibrated and half-calibrated power spectra 
given the calibrated power spectra and instrument response functions.
\item Calculates the optimal filter for a stochastic background
having $\Omega_{\rm gw}(f)=\Omega_0={\rm const}$ using the 
previously calculated information.
\end{enumerate}

\subsection*{Uses}
 
\begin{itemize}
\item \texttt{LALMalloc}
\item \texttt{LALFree}
\item \texttt{LALCreateVector}
\item \texttt{LALCCreateVector}
\item \texttt{LALSCreateVector}
\item \texttt{LALDestroyVector}
\item \texttt{LALCDestroyVector}
\item \texttt{LALSDestroyVector}
\item \texttt{LALSCoarseGrainFrequencySeries}
\item \texttt{LALCCoarseGrainFrequencySeries}
\item \texttt{LALStochasticOmegaGW}  
\item \texttt{LALStochasticInverseNoise}
\item \texttt{LALStochasticOptimalFilter}
\item \texttt{LALStochasticOptimalFilterNormalization}
\item \texttt{CastToReal4}
\item \texttt{CastToComplex8}
\end{itemize}

\subsection*{Notes}

\begin{itemize}
\item \texttt{LALConditionData} calls two static functions
\texttt{CastToReal4} and \texttt{CastToComplex8}, which are
used to cast time-series data, power spectral densities, 
and instrument response functions to the proper type for the 
LAL stochastic routines.
\item {Since no specific mechanism exists for reading the units string
    from an ILWD into the \texttt{LALUnit} structure, this DSO assumes
    its inputs have the expected units.}  These are
\begin{itemize}
\item Data Streams $h^{\scriptstyle{\rm W}}_{1,2}(t)$:  ADC counts
\item PSDs $P^{\scriptstyle{\rm W}}_{1,2}(f)$:
  (ADC counts)$^2$ / Hz
\item Response functions $\tilde{R}_{1,2}(f)$:
  ADC counts / attostrain = $10^{18}$ counts / strain
\end{itemize}
These are set ``by hand'' in \texttt{LALConditionData()} without
looking at the relevant \texttt{units} fields in the
\texttt{multiDimData}.  This should be changed to parse the
\texttt{units} fields and construct the appropriate \texttt{LALUnit}
structures, or at least check to see if it matches one of a set of
predefined strings.

\end{itemize}

\vfill{\footnotesize\input{StochasticConditionDataCV}}

*************************************************** </lalLaTeX> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <math.h>
#include <lal/LALStdlib.h>
#include <lal/LALConstants.h>
#include <lal/AVFactories.h>
#include <lal/DetectorSite.h>
#include <lal/Units.h>
#include <lal/Window.h>
#include <lal/Calibration.h>
#include <ExtractSeries.h>
#include <lal/CoarseGrainFrequencySeries.h>
#include "SGWB.h"
#include "SGWBConditionData.h"
#include "CastToFunctions.h"
#include "SGWBAuxFunctions.h"

NRCSID (SGWBCONDITIONDATAC, "$Id: SGWBConditionData.c,v 1.7 2003/09/29 20:11:29 kaice Exp $");

#ifdef STOCHASTIC_SO_DEBUG
static void PrintLALDetector(LALDetector *detector);
#endif


/************************ <lalVerbatim file="StochasticConditionDataCP"> */
void LALConditionData (
		       LALStatus*          status,
		       LALSearchInput*     inout,
		       void*               searchParams,
		       LALMPIParams          *mpiParams
		       )
     /*********************************************** </lalVerbatim> */
{

  /* Taken from Finn and Lazzarini gr-qc/0104040 */
#define STOCHASTICCONDITIONDATAC_ALLEGROLON ( -( 91.0L + ( 10.0L + (43.766L/60.0L) ) / 60.0L) * LAL_PI_180 )
#define STOCHASTICCONDITIONDATAC_ALLEGROLAT ( 30.0L + ( 24.0L + (45.110L/60.0L) ) / 60.0L * LAL_PI_180  )
#define STOCHASTICCONDITIONDATAC_ALLEGROAZ ( -40.0L * LAL_PI_180 )

  UINT4                           detInd;

  SequenceIndices                 sI;

  detGeom                        *geometry;
    
  LALWindowParams                 windowParams;
  REAL4Vector                    *window;
  UINT4                           halfWin;
  REAL4                          *sPtr;
  REAL4                          *sStopPtr;

  /*     REAL4FrequencySeries            overlap; */
  LALFrDetector                   frDetector;
  LALDetectorPair                 detectorPair;
  OverlapReductionFunctionParameters overlapParams;
  StochasticSearchParams              *params = NULL; 

  INITSTATUS (status, "LALConditionData", SGWBCONDITIONDATAC);
  ATTATCHSTATUSPTR (status);

  /* 
   *
   * check and cast searchParams 
   *
   */
    
  ASSERT(searchParams,status,STOCHASTICCONDITIONDATAH_ENULL, 
	 STOCHASTICCONDITIONDATAH_MSGENULL);
    
  params = (StochasticSearchParams *) searchParams;

  /* 
   *
   * make sure conditioning and analysis of data is done on search master 
   *
   */
    
  if ( params->searchMaster )
    {
      /* return immediately */
      DETATCHSTATUSPTR (status);
      RETURN (status);
    } else { 

      /*
       *
       *  check the input 
       *
       */

      /* check that inout data exists */
      ASSERT (inout, 
	      status,
	      STOCHASTICCONDITIONDATAH_ENULL, 
	      STOCHASTICCONDITIONDATAH_MSGENULL);
            
      /* check that number of sequences is correct */ 
      /* :NOTE: once the detector data are included this will be 8 */
      /* Have to remove this to handle new calibration methods properly
	 if (inout->numberSequences != 6)
	 {
	 ABORT(status, 
	 STOCHASTICCONDITIONDATAH_ENSEQ, 
	 STOCHASTICCONDITIONDATAH_MSGENSEQ);
	 }
      */
       
      /* check that multiDimData sequence data exists */
      if (!(inout->sequences))
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ENULL, 
		 STOCHASTICCONDITIONDATAH_MSGENULL);
	}
      
      /*
       *  
       * identify multiDimData sequences by ILWD comments, and check that 
       * the data has the correct type, dimensions, and length 
       *
       */
    
      TRY( LALFindCheckMDSequences(status->statusPtr,
				   inout,
				   params,
				   &sI),
	   status );

      if (sI.data1Index < 0) {
	ABORT(status, 
	      STOCHASTICCONDITIONDATAH_ENSEQ, 
	      STOCHASTICCONDITIONDATAH_MSGENSEQ);
      }
      if (sI.data2Index < 0) {
	ABORT(status, 
	      STOCHASTICCONDITIONDATAH_ENSEQ, 
	      STOCHASTICCONDITIONDATAH_MSGENSEQ);
      }

      /* calculate duration of a segment in seconds */
      params->segmentDuration = (REAL8) (params->initParams->numPoints)
	* inout->sequences[sI.data1Index].range.dTime.timeStepSize;

      /* calculate the jobLength based on the length of each segment
	 and the number of segments (numSegments) defined by the user */
      params->jobLength = (UINT4) (params->segmentDuration) 
	* params->initParams->numSegments;

      /* determine the fftLengths */
      if (!(params->initParams->baseFreq))
	{   /*  Real data*/
	  params->fftLength = 
	    (UINT4)1.0/(params->initParams->deltaF *
			inout->sequences[sI.data1Index].range.dTime.timeStepSize);
	} /* end real data option */
      else 
	{   /*  Complex data*/
	  /* KTR not sure this is correct, fairly certain it is wrong
	     or at least unnecessary */
	  params->fftLength = 
	    (UINT4)1.0/(params->initParams->deltaF *
			inout->sequences[sI.data1Index].range.dTime.timeStepSize);
	}
  
      /*       params->fftLength =  */
      /* 	(UINT4) 1.0/(params->initParams->deltaF  */
      /* 		     * inout->sequences[sI.data1Index].range.dTime.timeStepSize); */
      /*       fprintf(stdout,"init len *** %d **** \n", params->fftLength); */
      /*       fflush(stdout); */




      /*  If MC */




      /* The following conditional means we only do this if we're
	 calculating the cross-correlaiton */
      if ( params->optimalFilter )
	{ /* cross-corr calculation? */
	  TRY( LALAssignDataPointers( status->statusPtr,
				      inout,
				      params,
				      sI),
	       status );
	}


      /* printf("****** %d ****** %d \n", params->jobLength, params->initParams->numSegments); */
    
      /* We need the window even if we're not cross-correlating, for
	 normalization purposes */
    
      /* First window */
      windowParams.type = params->initParams->windowType1;
      if ( params->initParams->windowTop1 == 0 
	   || windowParams.type == Rectangular )
	{
	  windowParams.length = params->initParams->numPoints;
	  TRY( LALWindow(status->statusPtr, params->window1, &windowParams),
	       status );
	}
      else
	{
	  windowParams.length = params->initParams->numPoints 
	    - params->initParams->windowTop1;
	  if ( windowParams.length )
	    {
	      window = NULL;
	      TRY( LALSCreateVector(status->statusPtr,
				    &window,
				    windowParams.length),
		   status );
	      TRY( LALWindow(status->statusPtr, window, &windowParams),
		   status );
	      /* The number of points in the first half of the window, before
		 the flat top */
	      halfWin = (windowParams.length + 1) / 2;
	      sPtr = 
		memcpy ( params->window1->data, window->data,
			 halfWin * sizeof(REAL4) );
	      if (sPtr != params->window1->data)
		{
		  ABORT (status, 
			 STOCHASTICCONDITIONDATAH_EALOC, 
			 STOCHASTICCONDITIONDATAH_MSGEALOC);
		}
	    }
	  else
	    {
	      halfWin = 0;
	    }

	  for (sPtr = params->window1->data+halfWin,
		 sStopPtr = sPtr + params->initParams->windowTop1;
	       sPtr < sStopPtr; ++sPtr)
	    {
	      *sPtr = 1.0;
	    }

	  if ( windowParams.length )
	    {
	      sPtr = 
		memcpy ( sStopPtr, window->data + halfWin, 
			 (windowParams.length-halfWin) * sizeof(REAL4) );
	      if (sPtr != sStopPtr)
		{
		  ABORT (status, 
			 STOCHASTICCONDITIONDATAH_EALOC, 
			 STOCHASTICCONDITIONDATAH_MSGEALOC);
		}
	      TRY( LALSDestroyVector(status->statusPtr,
				     &window),
		   status );
	    }
	}

      /* Second window */
      windowParams.type = params->initParams->windowType2;
      if ( params->initParams->windowTop2 == 0 
	   || windowParams.type == Rectangular )
	{
	  windowParams.length = params->initParams->numPoints;
	  TRY( LALWindow(status->statusPtr, params->window2, &windowParams),
	       status );
	}
      else
	{
	  windowParams.length = params->initParams->numPoints 
	    - params->initParams->windowTop2;

	  if ( windowParams.length )
	    {
	      window = NULL;
	      TRY( LALSCreateVector(status->statusPtr,
				    &window,
				    windowParams.length),
		   status );
	      TRY( LALWindow(status->statusPtr, window, &windowParams),
		   status );
	      /* The number of points in the first half of the window, before
		 the flat top */
	      halfWin = (windowParams.length + 1) / 2;
	      sPtr = 
		memcpy ( params->window2->data, window->data,
			 halfWin * sizeof(REAL4) );
	      if (sPtr != params->window2->data)
		{
		  ABORT (status, 
			 STOCHASTICCONDITIONDATAH_EALOC, 
			 STOCHASTICCONDITIONDATAH_MSGEALOC);
		}
	    }
	  else
	    {
	      halfWin = 0;
	    }

	  for (sPtr = params->window2->data+halfWin,
		 sStopPtr = sPtr + params->initParams->windowTop2;
	       sPtr < sStopPtr; ++sPtr)
	    {
	      *sPtr = 1.0;
	    }

	  if ( windowParams.length )
	    {
	      sPtr = 
		memcpy ( sStopPtr, window->data + halfWin, 
			 (windowParams.length-halfWin) * sizeof(REAL4) );
	      if (sPtr != sStopPtr)
		{
		  ABORT (status, 
			 STOCHASTICCONDITIONDATAH_EALOC, 
			 STOCHASTICCONDITIONDATAH_MSGEALOC);
		}
	      TRY( LALSDestroyVector(status->statusPtr,
				     &window),
		   status );
	    }
	}

     /* 
       *
       * calculate overlap reduction function 
       *
       */

      /* fill in OverlapReductionFunctionParameters structure */
      overlapParams.length = params->initParams->numFrequencies;
      overlapParams.f0     = params->initParams->f0;
      overlapParams.deltaF = params->initParams->deltaF;

      /* Determine type or ID of first detector */
      switch (params->initParams->siteID1)
	{
	case 'i': case 'b': /*  Get geometry from frame*/
	  geometry = inout->sequences[sI.data1Index].range.dTime.geometry;
	  strncpy (frDetector.name,geometry->name, sizeof(frDetector.name));
	  /* LDAS and LAL now measure lat/lon in radians */
	  frDetector.vertexLongitudeRadians = geometry->longitude;
	  frDetector.vertexLatitudeRadians = geometry->latitude;
	  frDetector.vertexElevation = geometry->elevation;
	  /* LDAS and LAL now measure azimuth clockwise from North; */
	  frDetector.xArmAzimuthRadians = geometry->armXazimuth;
	  frDetector.xArmAltitudeRadians = geometry->armXaltitude;
	  frDetector.yArmAzimuthRadians = geometry->armYazimuth;
	  frDetector.yArmAltitudeRadians = geometry->armYaltitude;
	  switch (params->initParams->siteID1)
	    {
	    case 'i':
	      TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorOne),
				      &frDetector, LALDETECTORTYPE_IFODIFF ),
		   status );
	      break;
	    case 'b':
	      TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorOne),
				      &frDetector, LALDETECTORTYPE_CYLBAR ),
		   status );
	      break;
	    }
	  break;
	case 'H': /*  LIGO Hanford*/
	  detectorPair.detectorOne = lalCachedDetectors[LALDetectorIndexLHODIFF];
	  if (params->initParams->azimuth1)
	    {
	      ABORT ( status,
		      STOCHASTICCONDITIONDATAH_ESTID, 
		      STOCHASTICCONDITIONDATAH_MSGESTID );
	    }
	  break;
	case 'L': /*  LIGO Livingston*/
	  detectorPair.detectorOne = lalCachedDetectors[LALDetectorIndexLLODIFF];
	  if (params->initParams->azimuth1)
	    {
	      ABORT ( status,
		      STOCHASTICCONDITIONDATAH_ESTID, 
		      STOCHASTICCONDITIONDATAH_MSGESTID );
	    }
	  break;
	case 'G': /*  GEO-600*/
	  detectorPair.detectorOne
	    = lalCachedDetectors[LALDetectorIndexGEO600DIFF];
	  if (params->initParams->azimuth1)
	    {
	      ABORT ( status,
		      STOCHASTICCONDITIONDATAH_ESTID, 
		      STOCHASTICCONDITIONDATAH_MSGESTID );
	    }
	  break;
	case 'A': /*  ALLEGRO*/
	  strncpy (frDetector.name, "ALLEGRO", sizeof(frDetector.name));
	  frDetector.vertexLongitudeRadians
	    = STOCHASTICCONDITIONDATAC_ALLEGROLON;
	  frDetector.vertexLatitudeRadians
	    = STOCHASTICCONDITIONDATAC_ALLEGROLAT;
	  frDetector.xArmAltitudeRadians = 0;
	  if (params->initParams->azimuth1)
	    {
	      frDetector.xArmAzimuthRadians
		= *(params->initParams->azimuth1) * LAL_PI_180;
	    }
	  else 
	    {
	      frDetector.xArmAzimuthRadians = STOCHASTICCONDITIONDATAC_ALLEGROAZ;
	    }
	  TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorOne),
				  &frDetector, LALDETECTORTYPE_CYLBAR ),
	       status );
	  break;
	case '0': case '1': case '2': case '3': case '4':
	case '5': case '6': case '7': case '8': case '9':
	  if (params->initParams->azimuth1)
	    {
	      ABORT ( status,
		      STOCHASTICCONDITIONDATAH_ESTID, 
		      STOCHASTICCONDITIONDATAH_MSGESTID );
	    }
	  detInd = (UINT4) ctoi(params->initParams->siteID1);
	  /*  printf("%c %d\n",params->initParams->siteID1, detInd);*/
	  if ( detInd >= 0 && detInd < LALNumCachedDetectors )
	    {
	      detectorPair.detectorOne = lalCachedDetectors[detInd];
	      break;
	    }
	  /*  Fall through if out of range*/
	default:
	  ABORT ( status,
		  STOCHASTICCONDITIONDATAH_ESTID, 
		  STOCHASTICCONDITIONDATAH_MSGESTID );
	}

      /* Determine type or ID of second detector */
      switch (params->initParams->siteID2)
	{
	case 'i': case 'b': /*  Get geometry from frame*/
	  geometry = inout->sequences[sI.data2Index].range.dTime.geometry;
	  strncpy (frDetector.name,geometry->name, sizeof(frDetector.name));
	  /* LDAS and LAL now measure lat/lon in radians */
	  frDetector.vertexLongitudeRadians = geometry->longitude;
	  frDetector.vertexLatitudeRadians = geometry->latitude;
	  frDetector.vertexElevation = geometry->elevation;
	  /* LDAS and LAL now measure azimuth clockwise from North; */
	  frDetector.xArmAzimuthRadians = geometry->armXazimuth;
	  frDetector.xArmAltitudeRadians = geometry->armXaltitude;
	  frDetector.yArmAzimuthRadians = geometry->armYazimuth;
	  frDetector.yArmAltitudeRadians = geometry->armYaltitude;
	  switch (params->initParams->siteID2)
	    {
	    case 'i':
	      TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorTwo),
				      &frDetector, LALDETECTORTYPE_IFODIFF ),
		   status );
	      break;
	    case 'b':
	      TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorTwo),
				      &frDetector, LALDETECTORTYPE_CYLBAR ),
		   status );
	      break;
	    }
	  break;
	case 'H': /*  LIGO Hanford*/
	  detectorPair.detectorTwo = lalCachedDetectors[LALDetectorIndexLHODIFF];
	  if (params->initParams->azimuth2)
	    {
	      ABORT ( status,
		      STOCHASTICCONDITIONDATAH_ESTID, 
		      STOCHASTICCONDITIONDATAH_MSGESTID );
	    }
	  break;
	case 'L': /*  LIGO Livingston*/
	  detectorPair.detectorTwo = lalCachedDetectors[LALDetectorIndexLLODIFF];
	  if (params->initParams->azimuth2)
	    {
	      ABORT ( status,
		      STOCHASTICCONDITIONDATAH_ESTID, 
		      STOCHASTICCONDITIONDATAH_MSGESTID );
	    }
	  break;
	case 'G': /*  GEO-600*/
	  detectorPair.detectorTwo 
	    = lalCachedDetectors[LALDetectorIndexGEO600DIFF];
	  if (params->initParams->azimuth2)
	    {
	      ABORT ( status,
		      STOCHASTICCONDITIONDATAH_ESTID, 
		      STOCHASTICCONDITIONDATAH_MSGESTID );
	    }
	  break;
	case 'A': /*  ALLEGRO*/
	  strncpy (frDetector.name, "ALLEGRO", sizeof(frDetector.name));
	  frDetector.vertexLongitudeRadians
	    = STOCHASTICCONDITIONDATAC_ALLEGROLON;
	  frDetector.vertexLatitudeRadians
	    = STOCHASTICCONDITIONDATAC_ALLEGROLAT;
	  frDetector.xArmAltitudeRadians = 0;
	  if (params->initParams->azimuth2)
	    {
	      frDetector.xArmAzimuthRadians
		= *(params->initParams->azimuth2) * LAL_PI_180;
	    }
	  else 
	    {
	      frDetector.xArmAzimuthRadians = STOCHASTICCONDITIONDATAC_ALLEGROAZ;
	    }
	  TRY( LALCreateDetector( status->statusPtr, &(detectorPair.detectorTwo),
				  &frDetector, LALDETECTORTYPE_CYLBAR ),
	       status );
	  break;
	case '0': case '1': case '2': case '3': case '4':
	case '5': case '6': case '7': case '8': case '9':
	  if (params->initParams->azimuth2)
	    {
	      ABORT ( status,
		      STOCHASTICCONDITIONDATAH_ESTID, 
		      STOCHASTICCONDITIONDATAH_MSGESTID );
	    }
	  detInd = (UINT4) ctoi(params->initParams->siteID2);
	  /*  printf("%c %d\n",params->initParams->siteID2, detInd);*/
	  if ( detInd >= 0 && detInd < LALNumCachedDetectors )
	    {
	      detectorPair.detectorTwo = lalCachedDetectors[detInd];
	      break;
	    }
	  /*  Fall through if out of range*/
	default:
	  ABORT ( status,
		  STOCHASTICCONDITIONDATAH_ESTID, 
		  STOCHASTICCONDITIONDATAH_MSGESTID );
	}

      /* calculate overlap reduction function values */
#ifdef STOCHASTIC_SO_DEBUG
      PrintLALDetector(&detectorPair.detectorOne);
      PrintLALDetector(&detectorPair.detectorTwo);
#endif
      TRY( LALOverlapReductionFunction (status->statusPtr,
					params->overlapReduction,
					&detectorPair,
					&overlapParams),
	   status );
    
    }

  /* normal exit */
  DETATCHSTATUSPTR (status);
  RETURN (status);
}

#ifdef STOCHASTIC_SO_DEBUG
static void
PrintLALDetector(LALDetector *detector)
{
  printf( "  = { { %.15e, %.15e, %.15e },\n", detector->location[0],
          detector->location[1], detector->location[2] );
  printf( "      { { %.15g, %.15g, %.15g },\n", detector->response[0][0],
          detector->response[0][1], detector->response[0][2]);
  printf( "        { %.15g, %.15g, %.15g },\n", detector->response[1][0],
          detector->response[1][1], detector->response[1][2]);
  printf( "        { %.15g, %.15g, %.15g } },\n", detector->response[2][0],
          detector->response[2][1], detector->response[2][2]);
  printf("      { \"%s\",\n", detector->frDetector.name);
  printf("        %.15g, %.15g, %.15g,\n",
         detector->frDetector.vertexLongitudeRadians,
         detector->frDetector.vertexLatitudeRadians,
         detector->frDetector.vertexElevation);
  printf("        %.15e, %.15g,\n",
         detector->frDetector.xArmAltitudeRadians,
         detector->frDetector.xArmAzimuthRadians);
  printf("        %.15e, %.15g } }\n",
         detector->frDetector.yArmAltitudeRadians,
         detector->frDetector.yArmAzimuthRadians);
  return;
}
#endif
